@@ -60,6 +60,8 @@ class FileUploader(threading.Thread):
         self.formwrap = formwrap
         self.excepterror = excepterror
         super(FileUploader, self).__init__()
+        if not hasattr(self, 'isAlive'):
+            self.isAlive = self.is_alive
 
     def run(self):
         try:
@@ -84,6 +86,8 @@ class FileDownloader(threading.Thread):
         self.savefile = savefile
         self.exc = None
         super(FileDownloader, self).__init__()
+        if not hasattr(self, 'isAlive'):
+            self.isAlive = self.is_alive
 
     def run(self):
         try:
@@ -92,9 +96,10 @@ class FileDownloader(threading.Thread):
             self.exc = e
 
 
-def get_upload_form(filename, data, formname, otherfields):
+def get_upload_form(filename, data, formname, otherfields, boundary=BND):
+    ffilename = filename.split('/')[-1]
     if not formname:
-        formname = filename
+        formname = ffilename
     try:
         return uploadforms[filename]
     except KeyError:
@@ -102,17 +107,23 @@ def get_upload_form(filename, data, formname, otherfields):
             data = data.read()
         except AttributeError:
             pass
-        form = (b'--' + BND
+        form = b''
+        for ofield in otherfields:
+            tfield = otherfields[ofield]
+            xtra=''
+            if isinstance(tfield, dict):
+                tfield = json.dumps(tfield)
+                xtra = '\r\nContent-Type: application/json'
+            form += (b'--' + boundary
+                     + '\r\nContent-Disposition: form-data; '
+                       'name="{0}"{1}\r\n\r\n{2}\r\n'.format(
+                           ofield, xtra, tfield).encode('utf-8'))
+        form += (b'--' + boundary
                 + '\r\nContent-Disposition: form-data; '
                   'name="{0}"; filename="{1}"\r\n'.format(
-                      formname, filename).encode('utf-8'))
+                      formname, ffilename).encode('utf-8'))
         form += b'Content-Type: application/octet-stream\r\n\r\n' + data
-        for ofield in otherfields:
-            form += (b'\r\n--' + BND
-                     + '\r\nContent-Disposition: form-data; '
-                       'name="{0}"\r\n\r\n{1}'.format(
-                           ofield, otherfields[ofield]).encode('utf-8'))
-        form += b'\r\n--' + BND + b'--\r\n'
+        form += b'\r\n--' + boundary + b'--\r\n'
         uploadforms[filename] = form
         return form
 
@@ -156,9 +167,11 @@ class SecureHTTPConnection(httplib.HTTPConnection, object):
             self.sock.close()
             self.sock = None
 
-    def dupe(self):
+    def dupe(self, timeout=None):
+        if timeout is None:
+            timeout = self.mytimeout
         return SecureHTTPConnection(self.thehost, self.theport, clone=self,
-                                    timeout=self.mytimeout)
+                                    timeout=timeout)
 
     def set_header(self, key, value):
         self.stdheaders[key] = value
@@ -266,6 +279,22 @@ class SecureHTTPConnection(httplib.HTTPConnection, object):
             return json.loads(body) if body else {}, rsp.status
         return body, rsp.status
 
+    def grab_rsp(self, url, data=None, referer=None, headers=None, method=None):
+        webclient = self.dupe()
+        if isinstance(data, dict):
+            data = json.dumps(data)
+        if data:
+            if not method:
+                method = 'POST'
+            webclient.request(method, url, data, referer=referer,
+                              headers=headers)
+        else:
+            if not method:
+                method = 'GET'
+            webclient.request(method, url, referer=referer, headers=headers)
+        rsp = webclient.getresponse()
+        return rsp
+
     def download(self, url, file):
         """Download a file to filename or file object
 
@@ -303,21 +332,30 @@ class SecureHTTPConnection(httplib.HTTPConnection, object):
                      the file.
         :return:
         """
+        boundary = base64.b64encode(os.urandom(54))[:70]
         if data is None:
             data = open(filename, 'rb')
         ulhdrs = self.stdheaders.copy()
         if formwrap:
             self._upbuffer = io.BytesIO(get_upload_form(
-                filename, data, formname, otherfields))
-            ulhdrs['Content-Type'] = b'multipart/form-data; boundary=' + BND
+                filename, data, formname, otherfields, boundary))
+            ulhdrs['Content-Type'] = b'multipart/form-data; boundary=' + boundary
             ulhdrs['Content-Length'] = len(uploadforms[filename])
             self.ulsize = len(uploadforms[filename])
         else:
-            curroff = data.tell()
-            data.seek(0, 2)
-            self.ulsize = data.tell()
-            data.seek(curroff, 0)
-            self._upbuffer = data
+            canseek = True
+            try:
+                curroff = data.tell()
+            except io.UnsupportedOperation:
+                canseek = False
+                databytes = data.read()
+                self.ulsize = len(databytes)
+                self._upbuffer = io.BytesIO(databytes)
+            if canseek:
+                data.seek(0, 2)
+                self.ulsize = data.tell()
+                data.seek(curroff, 0)
+                self._upbuffer = data
             ulhdrs['Content-Type'] = b'application/octet-stream'
             ulhdrs['Content-Length'] = self.ulsize
         webclient = self.dupe()
@@ -376,3 +414,4 @@ class SecureHTTPConnection(httplib.HTTPConnection, object):
         except httplib.CannotSendRequest:
             self.broken = True
             raise
+
